/*
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <gtk/gtk.h>

#include "xfbib-bibtex-field.h"
#include "xfbib-bibtex-value.h"

struct _XfbibBibtexField
{
	GObject parent;
	GObject *owner;
	gchar *variable;
	XfbibBibtexValue *value;
};

typedef struct _XfbibBibtexFieldClass
{
	GObjectClass parent;
} XfbibBibtexFieldClass;

static void xfbib_bibtex_field_class_init(XfbibBibtexFieldClass *klass);

static void xfbib_bibtex_field_init(XfbibBibtexField *instance);
static void xfbib_bibtex_field_finalize(GObject *obj);

G_DEFINE_TYPE(XfbibBibtexField, xfbib_bibtex_field, G_TYPE_OBJECT)

static void
xfbib_bibtex_field_class_init(XfbibBibtexFieldClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	object_class->finalize = xfbib_bibtex_field_finalize;
}

static void
xfbib_bibtex_field_init(XfbibBibtexField *instance)
{
	instance->variable = NULL;
	instance->value = xfbib_bibtex_value_new(G_OBJECT(instance));
}

static void
xfbib_bibtex_field_finalize(GObject *obj)
{
	g_free(XFBIB_BIBTEX_FIELD(obj)->variable);
	G_OBJECT_CLASS(xfbib_bibtex_field_parent_class)->finalize(obj);
}

XfbibBibtexField *
xfbib_bibtex_field_new(GObject *obj)
{
	XfbibBibtexField *field;
	field = g_object_new(XFBIB_TYPE_BIBTEX_FIELD, NULL);
	field->owner = obj;
	return field;
}


GObject *
xfbib_bibtex_field_get_owner(XfbibBibtexField *field)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_FIELD(field), NULL);
	return field->owner;
}
	
gboolean
xfbib_bibtex_field_parse(XfbibBibtexField *field, const gchar *str)
{
	gchar **split = NULL;
	g_return_val_if_fail(XFBIB_IS_BIBTEX_FIELD(field), FALSE);

	split = g_strsplit(str, "=", 2);

	if (split[1] == NULL) {
		/* Equal sign missing */
		g_warning(_("No equal sign found in definition of Field\n"));
		g_strfreev(split);
		return FALSE;
	}

	/* Variable */
	g_free(field->variable);
	g_strstrip(split[0]);
	field->variable = g_strdup(split[0]);

	/* Value */
	if (!xfbib_bibtex_value_parse(field->value, split[1])) {
		g_strfreev(split);
		return FALSE;
	}

	g_strfreev(split);
	return TRUE;
}

const gchar *
xfbib_bibtex_field_get_variable(XfbibBibtexField *field)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_FIELD(field), NULL);	
	return field->variable;
}

void 
xfbib_bibtex_field_set_variable(XfbibBibtexField *field, const gchar *str)
{
	g_return_if_fail(XFBIB_IS_BIBTEX_FIELD(field));
	
	g_free(field->variable);
	field->variable = g_strdup(str);
}

XfbibBibtexValue *
xfbib_bibtex_field_get_value(XfbibBibtexField *field)
{
	g_return_val_if_fail(XFBIB_IS_BIBTEX_FIELD(field), NULL);
	return field->value;
}

void 
xfbib_bibtex_field_set_value(XfbibBibtexField *field, XfbibBibtexValue *value)
{
	g_return_if_fail(XFBIB_IS_BIBTEX_FIELD(field));
	g_object_unref(field->value);
	field->value = value;
}

