/*
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <libxfce4ui/libxfce4ui.h>
#include <stdio.h>

#include "xfbib-entry-edit-dialog.h"
#include "xfbib-menu-bar.h"
#include "xfbib-bibtex-field.h"
#include "xfbib-bibtex-string.h"
#include "xfbib-bibtex.h"
#include "xfbib-file-io.h"
#include "xfbib-state.h"
#include "xfbib-list-store.h"

struct _XfbibMenuBar
{
	GtkMenuBar parent;
	GtkWidget *file;
	GtkWidget *entry;
	GtkWidget *variable;
	GtkWidget *view;
	GtkWidget *view_columns;
	GtkWidget *help;
};

static XfbibWindow *window;

typedef struct _XfbibMenuBarClass
{
	GtkMenuBarClass parent;
} XfbibMenuBarClass;

static void xfbib_menu_bar_class_init(XfbibMenuBarClass *klass);

static void xfbib_menu_bar_init(XfbibMenuBar *instance);
static void xfbib_menu_bar_finalize(GObject *obj);

static GObjectClass *xfbib_menu_bar_parent_class = NULL;


static void
cb_file_new_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	/* TODO: Implement this */
}

static void
cb_file_open_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	gchar *filename;

	state = xfbib_state_new();

	if ((filename = xfbib_file_io_open(NULL)) != NULL) {
		xfbib_state_set_filename(state, filename);
		xfbib_window_set_title(XFBIB_WINDOW(xfbib_state_get_window(state)), filename);
	}
}

static void
cb_file_save_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	state = xfbib_state_new();

	xfbib_file_io_save(xfbib_state_get_filename(state));
}

static void
cb_file_save_as_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	xfbib_file_io_save_as();
}

static void
cb_file_quit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	gtk_widget_destroy(GTK_WIDGET(window));
}

static void
cb_entry_add_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	XfbibBibtexEntry *new_entry;
	GtkTreeModel *model;
	GtkWidget *dialog;

	state = xfbib_state_new();
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(xfbib_window_get_tree_view(
					XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_ENTRY)));

	dialog = xfbib_entry_edit_dialog_new(NULL);

	for(;;) {
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
			if(xfbib_entry_edit_dialog_valid_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog))) {
				new_entry = xfbib_entry_edit_dialog_get_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog));
				xfbib_list_store_add(XFBIB_LIST_STORE(model), G_OBJECT(new_entry));
			}
		} else {
			break;
		}
	}
	gtk_widget_destroy(dialog);
}

static void
cb_entry_remove_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	GtkWidget *tree_view;
	GtkTreeSelection *selection;
	GtkTreeModel     *model;
	GtkTreeIter       iter;

	state = xfbib_state_new();
	tree_view = GTK_WIDGET(xfbib_window_get_tree_view(XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_ENTRY));

	/* This will only work in single or browse selection mode! */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		xfbib_list_store_remove(XFBIB_LIST_STORE(model), &iter);
	} else {
		xfce_dialog_show_error(NULL, NULL, _("No row has been selected for removal"));
	}
}

static void
cb_entry_edit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	XfbibBibtexEntry *entry, *new_entry;
	GtkWidget *tree_view, *dialog;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;

	state = xfbib_state_new();
	tree_view = GTK_WIDGET(xfbib_window_get_tree_view(XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_ENTRY));

	/* This will only work in single or browse selection mode! */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		entry = XFBIB_BIBTEX_ENTRY(xfbib_list_store_get(XFBIB_LIST_STORE(model), &iter));

		dialog = xfbib_entry_edit_dialog_new(entry);
		for(;;) {
			if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
				if(xfbib_entry_edit_dialog_valid_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog))) {
					new_entry = xfbib_entry_edit_dialog_get_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog));
					xfbib_list_store_replace(XFBIB_LIST_STORE(model), G_OBJECT(entry), G_OBJECT(new_entry));
				}
			} else {
				break;
			}
		}
		gtk_widget_destroy(dialog);
	} else {
		xfce_dialog_show_error(NULL, NULL, _("No row has been selected for edit"));
	}
}

static void
cb_variable_add_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	g_print("TODO: Implement 'cb_variable_add_activate'");
#if 0
	XfbibState *state;
	GtkTreeModel *model;
	XfbibBibtexString *new_string;
	GtkWidget *dialog;

	state = xfbib_state_new();
	model = gtk_tree_view_get_model(GTK_TREE_VIEW(xfbib_window_get_tree_view(
					XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_STRING)));
	dialog = xfbib_entry_edit_dialog_new(NULL);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
		new_string = xfbib_entry_edit_dialog_get_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog));
		xfbib_list_store_add(XFBIB_LIST_STORE(model), G_OBJECT(new_string));
	}
	gtk_widget_destroy(dialog);
#endif
}

static void
cb_variable_remove_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	XfbibState *state;
	GtkWidget *tree_view;
	GtkTreeSelection *selection;
	GtkTreeModel     *model;
	GtkTreeIter       iter;

	state = xfbib_state_new();
	tree_view = GTK_WIDGET(xfbib_window_get_tree_view(XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_STRING));

	/* This will only work in single or browse selection mode! */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		xfbib_list_store_remove(XFBIB_LIST_STORE(model), &iter);
	} else {
		xfce_dialog_show_error(NULL, NULL, _("No row has been selected for removal"));
	}
}

static void
cb_variable_edit_activate (GtkMenuItem *menuitem, gpointer user_data)
{
	g_print("TODO: Implement 'cb_variable_edit_activate'");
#if 0
	XfbibState *state;
	XfbibBibtexString *string, *new_string;
	GtkWidget *tree_view, *dialog;
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;

	state = xfbib_state_new();
	tree_view = GTK_WIDGET(xfbib_window_get_tree_view(XFBIB_WINDOW(xfbib_state_get_window(state)), TYPE_STRING));

	/* This will only work in single or browse selection mode! */
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		string = XFBIB_BIBTEX_STRING(xfbib_list_store_get(XFBIB_LIST_STORE(model), &iter));

		dialog = xfbib_entry_edit_dialog_new(string);
		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
			new_string = xfbib_entry_edit_dialog_get_entry(XFBIB_ENTRY_EDIT_DIALOG(dialog));
			xfbib_list_store_replace(XFBIB_LIST_STORE(model), G_OBJECT(string), G_OBJECT(new_string));
		}
		gtk_widget_destroy(dialog);
	} else {
		xfce_err(_("No row has been selected for edit"));
	}
#endif
}

static void
cb_help_about_activate (GtkMenuItem *menuitem, gpointer ignored)
{
	gchar* xfbib_authors[] = {N_("David Gustafsson <david.s.gustafsson@gmail.com>"),
		N_("Jesper Karlsson <zarper@gmail.com>"),
		NULL};

/*
 * From the GtkAboutDialog documentation:
 * Note that prior to GTK+ 2.12, the "program-name" property was called "name".
 * This was changed to avoid the conflict with the "name" property.
 */
	gtk_show_about_dialog(GTK_WINDOW(window),
			"authors", xfbib_authors,
			"comments", _("Xfbib is a BibTeX editor for the Xfce Desktop Environment."),
			"copyright", "Copyright \302\251 2008 Jesper Karlsson & David Gustafsson",
			"license", XFCE_LICENSE_GPL,
			"logo-icon-name", PACKAGE,
#if GTK_CHECK_VERSION(2,12,0)
			"program-name", PACKAGE_NAME,
#else
			"name", PACKAGE_NAME,
#endif
			"title", "About " PACKAGE_NAME,
			"version", VERSION,
			"website", WEBSITE,
			NULL);
}

GType
xfbib_menu_bar_get_type (void)
{
	static GType type = 0;
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof (XfbibMenuBarClass),
			NULL,   /* base_init */
			NULL,   /* base_finalize */
			(GClassInitFunc) xfbib_menu_bar_class_init,   /* class_init */
			NULL,   /* class_finalize */
			NULL,   /* class_data */
			sizeof (XfbibMenuBar),
			0,      /* n_preallocs */
			(GInstanceInitFunc) xfbib_menu_bar_init    /* instance_init */
		};
		type = g_type_register_static (GTK_TYPE_MENU_BAR, "XfbibMenuBar", &info, 0);
	}
	return type;
}

static void
xfbib_menu_bar_class_init(XfbibMenuBarClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;

	xfbib_menu_bar_parent_class = g_type_class_peek_parent (klass);

	object_class->finalize = xfbib_menu_bar_finalize;
}

static void
xfbib_menu_bar_init(XfbibMenuBar *instance)
{
	GtkWidget *item;
	GtkAccelGroup *accel;

	accel = gtk_accel_group_new();

	/* File menu */
	instance->file = gtk_menu_new();
	/* Add things to menu */

	/* New */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_NEW, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->file), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_file_new_activate), "new");

	/* Open */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_OPEN, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->file), item);
	g_signal_connect (G_OBJECT (item), "activate",
			G_CALLBACK (cb_file_open_activate), NULL);

	/* Save */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->file), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_file_save_activate), "save");

	/* Save as */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_SAVE_AS, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->file), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_file_save_as_activate), "save_as");

	/* Quit */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->file), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_file_quit_activate), instance);

	item = gtk_menu_item_new_with_label(_("File"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->file);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance), item);

	/* Entry menu */
	instance->entry = gtk_menu_new();
	/* Add things to menu */
	/* Add */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->entry), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_entry_add_activate), instance);
	/* Remove */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->entry), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_entry_remove_activate), instance);

	/* Edit */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->entry), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_entry_edit_activate), instance);


	item = gtk_menu_item_new_with_label(_("Entry"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->entry);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance), item);

	/* Variable menu */
	instance->variable = gtk_menu_new();
	/* Add things to menu */
	/* Add */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->variable), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_variable_add_activate), instance);
	/* Remove */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->variable), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_variable_remove_activate), instance);

	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->variable), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_variable_edit_activate), instance);


	item = gtk_menu_item_new_with_label(_("Variable"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->variable);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance), item);

	/* View menu */
	instance->view = gtk_menu_new();
	/* Add things to menu */
	instance->view_columns = gtk_menu_new();
	/*
	item = gtk_menu_item_new_with_label("Show Columns");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->view_columns);
	gtk_menu_shell_append(GTK_MENU_SHELL (instance->view), item);
	*/

	/* Columns submenu */
	/*
	for (i = 0; i < XFBIB_BIBTEX_FIELD_N_FIELDS; i++) {
		str = g_strdup_printf("Show %s", xfbib_field_get_column_name(i));
		item = gtk_check_menu_item_new_with_mnemonic(str);
		g_free(str);
		gtk_menu_shell_append(GTK_MENU_SHELL (instance->view_columns), item);
	}
	*/

	item = gtk_check_menu_item_new_with_mnemonic(_("Show Toolbar"));
	gtk_menu_shell_append(GTK_MENU_SHELL (instance->view), item);

	item = gtk_check_menu_item_new_with_mnemonic(_("Show Status Bar"));
	gtk_menu_shell_append(GTK_MENU_SHELL (instance->view), item);

	item = gtk_menu_item_new_with_label(_("View"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->view);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance), item);

	/* Help menu */
	instance->help = gtk_menu_new();
	/* Add things to menu */
	item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, accel);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance->help), item);
	g_signal_connect_swapped (G_OBJECT (item), "activate",
			G_CALLBACK (cb_help_about_activate), "about");

	/* Create the button on the menu bar */
	item = gtk_menu_item_new_with_label(_("Help"));
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), instance->help);
	gtk_menu_shell_append(GTK_MENU_SHELL(instance), item);

}

static void
xfbib_menu_bar_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_menu_bar_parent_class)->finalize(obj);
}

GtkWidget *
xfbib_menu_bar_new(XfbibWindow *w)
{
	XfbibMenuBar *menu_bar;
	menu_bar = g_object_new(XFBIB_TYPE_MENU_BAR, NULL);
	window = w;
	return GTK_WIDGET(menu_bar);
}

