/* 
 * Copyright (c) 2008 Jesper Karlsson & David Gustafsson
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <string.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <libxfce4ui/libxfce4ui.h>

#include "xfbib-string.h"

struct _XfbibString
{
	GObject parent;
	gchar separator;
	gchar *str;
};

typedef struct _XfbibStringClass
{
	GObjectClass parent;
} XfbibStringClass;

static void xfbib_string_class_init(XfbibStringClass *klass);

static void xfbib_string_init(XfbibString *instance);
static void xfbib_string_finalize(GObject *obj);

G_DEFINE_TYPE(XfbibString, xfbib_string, G_TYPE_OBJECT)

static void
xfbib_string_class_init(XfbibStringClass *klass)
{
	GObjectClass *object_class = (GObjectClass *)klass;
	object_class->finalize = xfbib_string_finalize;
}

static void
xfbib_string_init(XfbibString *instance)
{
	instance->separator = '\0';
	instance->str = NULL;
}

static void
xfbib_string_finalize(GObject *obj)
{
	G_OBJECT_CLASS(xfbib_string_parent_class)->finalize(obj);
}

XfbibString *
xfbib_string_new()
{
	XfbibString *string;
	string = g_object_new(XFBIB_TYPE_STRING, NULL);
	return string;
}

gboolean
xfbib_string_parse(XfbibString *string, const gchar *str)
{
	string->separator = str[0];
	string->str = xfbib_string_parse_commands(g_strndup(str+1, strlen(str)-2));
	return TRUE;
}

gchar *
xfbib_string_get(XfbibString *string)
{
	return string->str;
}

void
xfbib_string_set(XfbibString *string, const gchar *str)
{
	g_free(string->str);
	g_print("Setting string: <%s>\n", str);
	string->str = g_strdup(str);
}

gchar
xfbib_string_get_separator(XfbibString *string)
{
	/* TODO: fix a xfbib_string_set_separator function */
	if(string->separator == '\0') {
		return '{';
	} else {
		return string->separator;
	}
}

gchar *
xfbib_string_parse_commands(gchar *str)
{
	int len, old_len, n, i;
	gchar *old_str;

	len = old_len = strlen(str) + 1;
	old_str = g_strdup(str);

	n = i = 0;
	while(n < len) {
		switch(old_str[i]) {
			case '\\':
				switch(old_str[i+1]) {
					case '&':
					case '%':
						len--;
						str = realloc(str, len);
						i++;
						str[n] = old_str[i];
						i++;
						n++;
						break;
					default:
						str[n] = old_str[i];
						i++;
						n++;
				}
				break;
			case '\n':
				len--;
				str = realloc(str, len);
				i++;
				break;
			case '\t':
				str[n] = ' ';
				i++;
				n++;
				break;
			default:
				str[n] = old_str[i];
				i++;
				n++;
		}
	}
	free(old_str);
	return str;	
}
